use std::path::PathBuf;

use actix_web::{
	web, get, post,
	Responder, HttpResponse, HttpRequest, HttpMessage
};

use crate::lib::{
	boards::*,
	users::UserCookie
};
use crate::api::types::{AppState, BoardForm};
//use types::{AppState, BoardForm};

#[get("/boards/list")]
pub async fn list(
	data: web::Data<AppState>
) -> impl Responder {
	HttpResponse::Ok().body(serde_json::to_string_pretty(&data.boards.lock().unwrap().clone()).unwrap())
}

#[post("/boards/new")]
pub async fn new(
	req: HttpRequest,
	data: web::Data<AppState>,
	form: web::Form<BoardForm>
) -> impl Responder {
	let cookie = req.cookie("auth");
	if cookie.is_none() {
		return HttpResponse::Unauthorized().body("Unauthorized: You are not logged in");
	}

	let cookie = match UserCookie::parse(cookie.unwrap().value()) {
		Ok(a) => a,
		Err(_e) => { return HttpResponse::BadRequest().body("Bad Request: Could not parse user cookie") }
	};

	let um = data.user_manager.lock().unwrap();

	if let Some(u) = um.get_user_by_id(cookie.id) {
		if cookie.secret != u.secret {
			return HttpResponse::BadRequest().body("Bad Request: Could not authenticate user");
		}
	} else {
		return HttpResponse::BadRequest().body("Bad Request: User does not exist");
	}

	let mut bf = data.board_factory.lock().unwrap();
	let mut path = PathBuf::from(
		format!(
			"{}/board.json", bf.next_id
		)
	);
	path.push(&form.name);
	if let Ok(_a) = Board::open(path) {
		return HttpResponse::Conflict().body("Conflict: Board already exists");
	}
	match bf.create_board(&form.name, &form.description) {
		Ok(a) => {
			if form.name.contains(' ') {
				return HttpResponse::BadRequest().body("Bad Request: Name cannot contain spaces");
			}
			println!("new board");
			data.boards.lock().unwrap().push(a.clone());
			HttpResponse::Created().body(serde_json::to_string(&a).unwrap())
		},
		Err(e) => {
			let a = format!("Internal Server Error: {}", e);
			HttpResponse::InternalServerError().body(a)
		}
	}
}
