pub mod boards;
pub mod messages;
pub mod users;

use boards::BoardFactory;
use messages::MessageFactory;
use users::UserManager;

use std::{
	path::Path
};

use serde::{de::{DeserializeOwned}, Serialize};

pub trait Archiveable<T>: Serialize where T: DeserializeOwned {
	fn get_json(&self) -> String where Self: Serialize {
		serde_json::to_string(self).unwrap()
	}

	fn get_root(&self) -> &Path;

	fn save_state<P>(
		&self, path: P
	) -> Result<(), std::io::Error> where
		Self: Serialize,
		P: AsRef<Path>
	{
		std::fs::write({
			let mut p = self.get_root().to_path_buf();
			p.push(path);
			p
		}, self.get_json())
	}

	fn load_state<P>(
		path: P
	) -> Result<T, std::io::Error>
	where
		P: AsRef<Path>,
	{
		let t = std::fs::read_to_string(path)?;
		let a = serde_json::from_str::<T>(&t);
		Ok(a.unwrap())
		//Err(std::io::Error::new(std::io::ErrorKind::NotFound, "asd"))
	}
}

pub struct FactoryFactory {
	data_dir: String
}

impl FactoryFactory {
	pub fn new<P>(dir: P) -> FactoryFactory where P: AsRef<Path> {
		FactoryFactory {
			data_dir: dir.as_ref().to_str().unwrap().to_string()
		}
	}

	pub fn build(&self) -> (BoardFactory, MessageFactory, UserManager) {
		(BoardFactory::new(self.data_dir.clone()),
		MessageFactory {
			next_id: 0,
			last_write: None,
			data_dir: self.data_dir.clone()
		},
		UserManager::new(self.data_dir.clone()))
	}
}
